==> walletpackage-1.1.0.tar.gz: lib/src/tronTransactioin.dart <==
import 'dart:ffi';
import 'dart:typed_data';

import 'package:crypton/crypton.dart';
import 'package:fixnum/fixnum.dart';
import 'package:crypto/crypto.dart';
import 'package:protobuf/protobuf.dart';
import 'package:walletpackage/src/protos/gen/google/protobuf/any.pb.dart';
import 'package:walletpackage/src/protos/gen/balance_contract.pb.dart';
import 'package:walletpackage/src/protos/gen/Tron.pb.dart';
import 'package:walletpackage/src/protos/gen/smart_contract.pb.dart';
import 'package:walletpackage/src/rlp.dart';
import 'package:web3dart/crypto.dart' as web3dart_crypto;
import 'package:bs58check/bs58check.dart';
import 'walletProxy.dart';

class Ttransaction {
  Transaction tx;

  Ttransaction(this.tx);
}

class TronTransaction {
  static Future<String> signTronTransaction(
      String privateKeyHex, Ttransaction transaction) async {
    if (transaction == null) {
      return null;
    } else {
      final Uint8List unsignedTx = transaction.tx.rawData.writeToBuffer();
      // print('unsignedTx: ' + bytesToHex(unsignedTx));

      Digest txid = sha256.convert(unsignedTx);
      // print('txid: ' + txid.toString());

      ECPrivateKey privateKey = ECPrivateKey.fromString(privateKeyHex);
      web3dart_crypto.MsgSignature ms = web3dart_crypto.sign(
          txid.bytes, web3dart_crypto.hexToBytes(privateKeyHex));
      String v = '0' + ms.v.toRadixString(16);
      String signature =
          web3dart_crypto.bytesToHex(web3dart_crypto.intToBytes(ms.r)) +
              web3dart_crypto.bytesToHex(web3dart_crypto.intToBytes(ms.s)) +
              v.substring(v.length - 2);
      // print('signature: ' + signature);

      transaction.tx.signature.add(web3dart_crypto.hexToBytes(signature));
      Uint8List signedTx = transaction.tx.writeToBuffer();
      // print('signedTx: ' + web3dart_crypto.bytesToHex(signedTx));
      return web3dart_crypto.bytesToHex(signedTx);
    }
  }

  static Future<Ttransaction> createTrc20Transaction(String from, String to,
      num value, String refBlockHash, String refBlockNum, int feeLimit,
      {String type = 'TRCUSDT'}) async {
    if (WalletProxy.trc20Type.keys.contains(type)) {
      CoinInfo coinInfo = WalletProxy.trc20Type[type];
      TriggerSmartContract tsc = TriggerSmartContract.create();
      tsc.ownerAddress = _decode58Check(from);
      tsc.contractAddress = _decode58Check(coinInfo.address);

      String operationHex = 'a9059cbb0000000000000000000000' +
          web3dart_crypto.bytesToHex(_decode58Check(to));
      String valueHex =
          numPow2BigInt(value, coinInfo.decimals).toRadixString(16);
      String tokenHex =
          ('0000000000000000000000000000000000000000000000000000000000000000' +
                  valueHex)
              .substring(valueHex.length);
      String dataHex = operationHex + tokenHex;
      // print(dataHex);
      tsc.data = web3dart_crypto.hexToBytes(dataHex);

      Transaction tx = _createTronTransaction(
          tsc,
          Transaction_Contract_ContractType.TriggerSmartContract,
          refBlockHash,
          refBlockNum,
          feeLimit);
      return Ttransaction(tx);
    }
    return null;
  }

  static Future<Ttransaction> createTrxTransaction(String from, String to,
      num value, String refBlockHash, String refBlockNum, int feeLimit,
      {String type = 'TRX'}) async {
    if (WalletProxy.tronType.keys.contains(type)) {
      CoinInfo coinInfo = WalletProxy.tronType[type];
      TransferContract tfc = TransferContract.create();
      tfc.ownerAddress = _decode58Check(from);
      tfc.toAddress = _decode58Check(to);
      tfc.amount = Int64(numPow2BigInt(value, coinInfo.decimals).toInt());

      Transaction tx = _createTronTransaction(
          tfc,
          Transaction_Contract_ContractType.TransferContract,
          refBlockHash,
          refBlockNum,
          feeLimit);
      return Ttransaction(tx);
    }
  }

  static Uint8List _decode58Check(String input) {
    Uint8List address = base58.decode(input);
    // print(address.length);
    if (address.length == 25) {
      // print(bytesToHex(address.sublist(0, 21)));
      return address.sublist(0, 21);
    }
    return null;
  }

  static Transaction _createTronTransaction(
      GeneratedMessage tfc,
      Transaction_Contract_ContractType ct,
      String refBlockHash,
      String refBlockNum,
      int feeLimit) {
    Transaction tx = Transaction.create();

    Transaction_Contract txc = Transaction_Contract.create();

    try {
      Any any = Any.pack(tfc);
      txc.parameter = any;
    } catch (e) {
      print('TronTransaction.createTronTransaction error : $e');
      return null;
    }
    txc.type = ct;
    Transaction_raw tr = tx.ensureRawData();
    tr.contract.add(txc);
    tr.timestamp = Int64(DateTime.now().millisecondsSinceEpoch);
    tr.expiration = tr.timestamp + 10 * 60 * 60 * 1000;

    // tr.timestamp = Int64(1611660032070);
    // tr.expiration = Int64(1611696032070);

    tx.rawData.refBlockHash =
        web3dart_crypto.hexToBytes(refBlockHash.substring(16, 32));
    tx.rawData.refBlockBytes =
        web3dart_crypto.hexToBytes(refBlockHash.substring(12, 16));
    tx.rawData.refBlockNum = Int64.parseInt(refBlockNum);
    tx.rawData.feeLimit = Int64(feeLimit);

    return tx;
  }
}

==> walletpackage-1.1.25.tar.gz: lib/src/tronTransactioin.dart <==
import 'dart:ffi';
import 'dart:typed_data';

import 'package:crypton/crypton.dart';
import 'package:fixnum/fixnum.dart';
import 'package:crypto/crypto.dart';
import 'package:protobuf/protobuf.dart';
import 'package:walletpackage/src/protos/gen/google/protobuf/any.pb.dart';
import 'package:walletpackage/src/protos/gen/balance_contract.pb.dart';
import 'package:walletpackage/src/protos/gen/Tron.pb.dart';
import 'package:walletpackage/src/protos/gen/smart_contract.pb.dart';
import 'package:walletpackage/src/rlp.dart';
import 'package:web3dart/crypto.dart' as web3dart_crypto;
import 'package:bs58check/bs58check.dart';
import 'walletProxy.dart';

class Ttransaction {
  Transaction tx;

  Ttransaction(this.tx);
}

class TronTransaction {
  static Future<String> signTronTransaction(
      String privateKeyHex, Ttransaction transaction) async {
    if (transaction == null) {
      return null;
    } else {
      final Uint8List unsignedTx = transaction.tx.rawData.writeToBuffer();
      // print('unsignedTx: ' + bytesToHex(unsignedTx));

      Digest txid = sha256.convert(unsignedTx);
      // print('txid: ' + txid.toString());

      ECPrivateKey privateKey = ECPrivateKey.fromString(privateKeyHex);
      web3dart_crypto.MsgSignature ms = web3dart_crypto.sign(
          txid.bytes, web3dart_crypto.hexToBytes(privateKeyHex));
      String v = '0' + ms.v.toRadixString(16);
      String signature =
          web3dart_crypto.bytesToHex(web3dart_crypto.intToBytes(ms.r)) +
              web3dart_crypto.bytesToHex(web3dart_crypto.intToBytes(ms.s)) +
              v.substring(v.length - 2);
      // print('signature: ' + signature);

      transaction.tx.signature.add(web3dart_crypto.hexToBytes(signature));
      Uint8List signedTx = transaction.tx.writeToBuffer();
      // print('signedTx: ' + web3dart_crypto.bytesToHex(signedTx));
      return web3dart_crypto.bytesToHex(signedTx);
    }
  }

  static Future<Ttransaction> createTrc20Transaction(
      String from, String to, num value, String refBlockHash, String refBlockNum,
      {String type = 'TRCUSDT'}) async {
    if (WalletProxy.trc20Type.keys.contains(type)) {
      CoinInfo coinInfo = WalletProxy.trc20Type[type];
      TriggerSmartContract tsc = TriggerSmartContract.create();
      tsc.ownerAddress = _decode58Check(from);
      tsc.contractAddress = _decode58Check(coinInfo.address);

      String operationHex =
          'a9059cbb0000000000000000000000' + web3dart_crypto.bytesToHex(_decode58Check(to));
      String valueHex =
      numPow2BigInt(value, coinInfo.decimals).toRadixString(16);
      String tokenHex =
      ('0000000000000000000000000000000000000000000000000000000000000000' +
          valueHex)
          .substring(valueHex.length);
      String dataHex = operationHex + tokenHex;
      // print(dataHex);
      tsc.data = web3dart_crypto.hexToBytes(dataHex);

      Transaction tx = _createTronTransaction(
          tsc,
          Transaction_Contract_ContractType.TriggerSmartContract,
          refBlockHash,
          refBlockNum,
          coinInfo.gasLimit);
      return Ttransaction(tx);
    }
    return null;
  }

  static Future<Ttransaction> createTrxTransaction(String from, String to,
      num value, String refBlockHash, String refBlockNum,
      {String type = 'TRX'}) async {
    if (WalletProxy.tronType.keys.contains(type)) {
      CoinInfo coinInfo = WalletProxy.tronType[type];
      TransferContract tfc = TransferContract.create();
      tfc.ownerAddress = _decode58Check(from);
      tfc.toAddress = _decode58Check(to);
      tfc.amount = Int64(numPow2BigInt(value, coinInfo.decimals).toInt());

      Transaction tx = _createTronTransaction(
          tfc,
          Transaction_Contract_ContractType.TransferContract,
          refBlockHash,
          refBlockNum,
          coinInfo.gasLimit);
      return Ttransaction(tx);
    }
  }

  static Uint8List _decode58Check(String input) {
    Uint8List address = base58.decode(input);
    // print(address.length);
    if (address.length == 25) {
      // print(bytesToHex(address.sublist(0, 21)));
      return address.sublist(0, 21);
    }
    return null;
  }

  static Transaction _createTronTransaction(
      GeneratedMessage tfc,
      Transaction_Contract_ContractType ct,
      String refBlockHash,
      String refBlockNum,
      int feeLimit) {
    Transaction tx = Transaction.create();

    Transaction_Contract txc = Transaction_Contract.create();

    try {
      Any any = Any.pack(tfc);
      txc.parameter = any;
    } catch (e) {
      print('TronTransaction.createTronTransaction error : $e');
      return null;
    }
    txc.type = ct;
    Transaction_raw tr = tx.ensureRawData();
    tr.contract.add(txc);
    tr.timestamp = Int64(DateTime.now().millisecondsSinceEpoch);
    tr.expiration = tr.timestamp + 10 * 60 * 60 * 1000;

    // tr.timestamp = Int64(1611660032070);
    // tr.expiration = Int64(1611696032070);

    tx.rawData.refBlockHash =
        web3dart_crypto.hexToBytes(refBlockHash.substring(16, 32));
    tx.rawData.refBlockBytes =
        web3dart_crypto.hexToBytes(refBlockHash.substring(12, 16));
    tx.rawData.refBlockNum = Int64.parseInt(refBlockNum);
    tx.rawData.feeLimit = Int64(feeLimit);

    return tx;
  }
}
